﻿-------------------------------------------------------------------------------
-- GodSendPlus v1.2.3
-------------------------------------------------------------------------------
--
-- Original code Copyright (c) 2015 Tim Billington (Focus) (MIT License)
-- Additional code Copyright (c) 2015-2017 @nooblybear (MIT License)
--
-- This Add-on is not created by, affiliated with or sponsored by ZeniMax Media
-- Inc. or its affiliates. The Elder Scrolls® and related logos are registered
-- trademarks or trademarks of ZeniMax Media Inc. in the United States and/or
-- other countries.
--
-- You can read the full terms at:
-- https://account.elderscrollsonline.com/add-on-terms
--
-------------------------------------------------------------------------------

-- Local Constants

local ADDON_NAME = "GodSendPlus"
local ADDON_VERSION = "1.2.4"
local SAVEDVARS_NAME = "GodSendPlus_SavedVars"
local SAVEDVARS_VERSION = 1

local GODSENDPLUS_DEBUG = false
local GODSENDPLUS_ACTIVE = false
local GODSENDPLUS_PAUSED = false

local GODSENDPLUS_WINDOW = nil
local GODSENDPLUS_COMPOSE = nil
local GODSENDPLUS_RECIPIENTS = nil
local GODSENDPLUS_INBOX = nil
local GODSENDPLUS_SENT = nil
local GODSENDPLUS_IMPORT = nil

local RECIPIENTS_SORT_KEYS =
{
	["id"] = { isNumeric = true },
	["name"] = { tiebreaker = "id" },
	["rank"] = { tiebreaker = "id" },
	["mail"] = { tiebreaker = "id" },
	["rankIndex"] = { tiebreaker = "id", isNumeric = true }
}

-- Local Variables

local GodSendPlusWindow = ZO_Object:Subclass()
local GodSendPlusCompose = ZO_Object:Subclass()
local GodSendPlusRecipients = ZO_SortFilterList:Subclass()
local GodSendPlusInbox = ZO_SortFilterList:Subclass()
local GodSendPlusSent = ZO_SortFilterList:Subclass()
local GodSendPlusImport = ZO_SortFilterList:Subclass()

local origPlayerContextMenu = nil

local userImportList = {}
local userImportMode = false
local _onrs

local Guilds = 
{
	["GuildNames"] = { },
	["GuildRanks"] = { },
	["GuildMembers"] = { }
}

local GuildChoice = {}
local CurrentMail = {}
local PendingMail = {}

local lastKnownRecipient = nil
local mailBoxOpen = false
local progressBarTotal = 0
local progressBarUnit = 0
local sentMailCount = 0
local recipientID = 0
local throttleTimer = 2000

local DefaultVars = {}
local SavedVars

-- Menu Buttons

local menuBarButtons =
{
	{
		categoryName = SI_GODSENDPLUS_TITLE_IMPORT,
		descriptor = "GodSendPlusImport",
		normal = "esoui/art/bank/bank_tabicon_withdraw_up.dds",
		pressed = "esoui/art/bank/bank_tabicon_withdraw_down.dds",
		highlight = "esoui/art/bank/bank_tabicon_withdraw_over.dds",
	},
	{
		categoryName = SI_GODSENDPLUS_TITLE_COMPOSE,
		descriptor = "GodSendPlusCompose",
		normal = "esoui/art/mail/mail_tabicon_compose_up.dds",
		pressed = "esoui/art/mail/mail_tabicon_compose_down.dds",
		highlight = "esoui/art/mail/mail_tabicon_compose_over.dds",
	},
	{
		categoryName = SI_GODSENDPLUS_TITLE_RECIPIENTS,
		descriptor = "GodSendPlusRecipients",
		normal = "esoui/art/contacts/tabicon_friends_up.dds",
		pressed = "esoui/art/contacts/tabicon_friends_down.dds",
		highlight = "esoui/art/contacts/tabicon_friends_over.dds",
	},
}

-------------------------------------------------------------------------------
-- TEXT HANDLERS
-------------------------------------------------------------------------------

-- Char Counter

local function CharCounter(str)
    if not str then return 0 end
    local count = 0 
    for i = 1, #str do
        if string.byte(str, i) then
            count = count + 1 
        end
    end 
    return count
end

-- Convert Color

local function hex2rgb(hex)
    hex = hex:gsub("#","")
    return tonumber("0x"..hex:sub(1,2),16)/255, tonumber("0x"..hex:sub(3,4),16)/255, tonumber("0x"..hex:sub(5,6),16)/255
end

-------------------------------------------------------------------------------
-- CONTROL WINDOW
-------------------------------------------------------------------------------
--
--

function GodSendPlusWindow:New(control)
	local manager = ZO_Object.New(self)
	
	manager.control = control
	manager.menuBar = GetControl(control, "MenuBar")
	manager.menuBarLabel = GetControl(control, "MenuBarLabel")
	manager.body = GetControl(control, "Body")
	manager.visible = false
	manager.activeTab = "GodSendPlusCompose"
	manager.labels = {}
	manager.tabs = {}
    control.container = manager
	
	for _, buttonData in ipairs(menuBarButtons) do
		buttonData.callback = function() manager:SetTab(buttonData.descriptor) end
		ZO_MenuBar_AddButton(manager.menuBar, buttonData)
		ZO_MenuBar_SetDescriptorEnabled(manager.menuBar, buttonData.descriptor, true)
		manager.labels[buttonData.descriptor] = GetString(buttonData.categoryName)
		manager.tabs[buttonData.descriptor] = CreateControlFromVirtual(buttonData.descriptor, manager.body, buttonData.descriptor)
	end
	
	ZO_MenuBar_SelectDescriptor(manager.menuBar, manager.activeTab)
	
    return manager
end

-- Window Tab Handlers

function GodSendPlusWindow:SetTab(tabName)
	self.menuBarLabel:SetText(self.labels[tabName])
	self.menuBarLabel:SetAnchor(TOPRIGHT, GodSendPlusWindowMenuBar, TOPRIGHT, -170 , 4)
	self.menuBarLabel:SetFont("ZoFontWinH3")
	self.tabs[self.activeTab]:SetHidden(true)
	self.tabs[tabName]:SetHidden(false)
	self.activeTab = tabName
end

function GodSendPlusWindow:ShowTab(tabName, skipSound)
	if (not self.visible) then
		self.visible = true
		self.control:SetHidden(false)
		SCENE_MANAGER:SetInUIMode(true)
		if (not skipSound) then PlaySound(SOUNDS.SYSTEM_WINDOW_OPEN) end
	end
	
	if (self.activeTab ~= tabName) then
		if (not ZO_MenuBar_SelectDescriptor(self.menuBar, tabName, true)) then
			if (GODSENDPLUS_DEBUG) then d("MenuBar Error: activeTab ~= tabName. Selected 1st visible tab.") end
			ZO_MenuBar_SelectFirstVisibleButton(self.menuBar, true)
		end
	end
end

function GodSendPlusWindow:Toggle()
    if (self.visible) then
        self:Hide()
    else
        self:ShowTab(self.activeTab)
    end
end

function GodSendPlusWindow:ToggleTab(tabName)
    if (self.visible and self.activeTab == tabName) then
        self:Hide()
    else
        self:ShowTab(tabName)
    end
end

function GodSendPlusWindow:Hide()
	if (self.visible) then
		self.visible = false
		self.control:SetHidden(true)
		SCENE_MANAGER:SetInUIMode(false)
		PlaySound(SOUNDS.SYSTEM_WINDOW_CLOSE)
	end
end

-------------------------------------------------------------------------------
-- CONTROL RECIPIENTS
-------------------------------------------------------------------------------

function GodSendPlusRecipients:New(control)
	local manager = ZO_SortFilterList.New(self, control)
	
	ZO_ScrollList_AddDataType(manager.list, 1, "GodSendPlusRecipientsRow", 30, function(control, data) manager:SetupRow(control, data) end)
	ZO_ScrollList_EnableHighlight(manager.list, "ZO_ThinListHighlight")
	
	manager:SetAlternateRowBackgrounds(true)
	
	manager.control = control
    control.container = manager
	manager.masterList = {}
	manager.sortHeaderGroup:SelectHeaderByKey("id")
	
	return manager
end

function GodSendPlusRecipients:SetupRow(control, data)
	ZO_SortFilterList.SetupRow(self, control, data)
	
	control:SetHandler("OnMouseUp", function(control, button, upInside, linkText) self:OnRowMouseUp(control, button, upInside, linkText) end)
	
	local playerLink = ("|H0:display:%s|h%s|h"):format(data.name, data.name)
	GetControl(control, "ID"):SetText(data.id)
	GetControl(control, "Name"):SetText(playerLink)
	GetControl(control, "Rank"):SetText(data.rank)
	GetControl(control, "Mail"):SetText(data.mail)
	
	GetControl(control, "ID"):SetHorizontalAlignment(TEXT_ALIGN_RIGHT)
end

function GodSendPlusRecipients:BuildMasterList()
	ZO_ClearNumericallyIndexedTable(self.masterList)
	
	for _, data in ipairs(Guilds.GuildMembers) do
		table.insert(self.masterList, data)
	end
end

function GodSendPlusRecipients:FilterScrollList()
    local scrollData = ZO_ScrollList_GetDataList(self.list)
    ZO_ClearNumericallyIndexedTable(scrollData)

    for i = 1, #self.masterList do
		local data = self.masterList[i]
		if data.mail ~= false then
		table.insert(scrollData, ZO_ScrollList_CreateDataEntry(1, data))
		else end
    end
end

function GodSendPlusRecipients:CompareRows(listEntry1, listEntry2)
	return ZO_TableOrderingFunction(listEntry1.data, listEntry2.data, self.currentSortKey, RECIPIENTS_SORT_KEYS, self.currentSortOrder)
end

function GodSendPlusRecipients:SortScrollList()
	local scrollData = ZO_ScrollList_GetDataList(self.list)
	
	if self.currentSortKey == "rank" then
		self.currentSortKey = "rankIndex"
	end
	
	table.sort(scrollData, function(listEntry1, listEntry2) return self:CompareRows(listEntry1, listEntry2) end)
end

function GodSendPlusRecipients:ColorRow(control, data, mouseIsOver)

end

function GodSendPlusRecipients:OnRowMouseUp(control, button, upInside, linkText)
	if (button == 2 and linkText) then
		if (not self.unlockSelectionCallback) then self.unlockSelectionCallback = function() self:UnlockSelection() end	end
		SetMenuHiddenCallback(self.unlockSelectionCallback)
		self:LockSelection()
	end
end

-------------------------------------------------------------------------------
-- CONTROL INBOX
-------------------------------------------------------------------------------

function GodSendPlusInbox:New(control)
	local manager = ZO_SortFilterList.New(self, control)
	
	ZO_ScrollList_AddDataType(manager.list, 1, "GodSendPlusInboxRow", 30, function(control, data) manager:SetupRow(control, data) end)
	ZO_ScrollList_EnableHighlight(manager.list, "ZO_ThinListHighlight")
	
	--manager:SetAlternateRowBackgrounds(true)
	
	manager.control = control
    control.container = manager
	manager.masterList = {}
	
	return manager
end

function GodSendPlusInbox:SetupRow(control, data)
	ZO_SortFilterList.SetupRow(self, control, data)
	
	control:SetHandler("OnMouseUp", function(control, button, upInside, linkText) self:OnRowMouseUp(control, button, upInside, linkText) end)
end

function GodSendPlusInbox:BuildMasterList()
	ZO_ClearNumericallyIndexedTable(self.masterList)
end

function GodSendPlusInbox:FilterScrollList()
    local scrollData = ZO_ScrollList_GetDataList(self.list)
    ZO_ClearNumericallyIndexedTable(scrollData)
end

function GodSendPlusInbox:CompareRows(listEntry1, listEntry2)
	return ZO_TableOrderingFunction(listEntry1.data, listEntry2.data, self.currentSortKey, RECIPIENTS_SORT_KEYS, self.currentSortOrder)
end

function GodSendPlusInbox:SortScrollList()
	local scrollData = ZO_ScrollList_GetDataList(self.list)
	
	table.sort(scrollData, function(listEntry1, listEntry2) return self:CompareRows(listEntry1, listEntry2) end)
end

function GodSendPlusInbox:ColorRow(control, data, mouseIsOver)

end

function GodSendPlusInbox:OnRowMouseUp(control, button, upInside, linkText)
	if (button == 2 and linkText) then
		if (not self.unlockSelectionCallback) then self.unlockSelectionCallback = function() self:UnlockSelection() end	end
		SetMenuHiddenCallback(self.unlockSelectionCallback)
		self:LockSelection()
	end
end

-------------------------------------------------------------------------------
-- CONTROL SENT
-------------------------------------------------------------------------------

function GodSendPlusSent:New(control)
	local manager = ZO_SortFilterList.New(self, control)
	
	ZO_ScrollList_AddDataType(manager.list, 1, "GodSendPlusSentRow", 30, function(control, data) manager:SetupRow(control, data) end)
	ZO_ScrollList_EnableHighlight(manager.list, "ZO_ThinListHighlight")
	
	--manager:SetAlternateRowBackgrounds(true)
	
	manager.control = control
    control.container = manager
	manager.masterList = {}
	
	return manager
end

function GodSendPlusSent:SetupRow(control, data)
	ZO_SortFilterList.SetupRow(self, control, data)
	
	control:SetHandler("OnMouseUp", function(control, button, upInside, linkText) self:OnRowMouseUp(control, button, upInside, linkText) end)
end

function GodSendPlusSent:BuildMasterList()
	ZO_ClearNumericallyIndexedTable(self.masterList)
end

function GodSendPlusSent:FilterScrollList()
    local scrollData = ZO_ScrollList_GetDataList(self.list)
    ZO_ClearNumericallyIndexedTable(scrollData)
end

function GodSendPlusSent:CompareRows(listEntry1, listEntry2)
	return ZO_TableOrderingFunction(listEntry1.data, listEntry2.data, self.currentSortKey, RECIPIENTS_SORT_KEYS, self.currentSortOrder)
end

function GodSendPlusSent:SortScrollList()
	local scrollData = ZO_ScrollList_GetDataList(self.list)
	
	table.sort(scrollData, function(listEntry1, listEntry2) return self:CompareRows(listEntry1, listEntry2) end)
end

function GodSendPlusSent:ColorRow(control, data, mouseIsOver)

end

function GodSendPlusSent:OnRowMouseUp(control, button, upInside, linkText)
	if (button == 2 and linkText) then
		if (not self.unlockSelectionCallback) then self.unlockSelectionCallback = function() self:UnlockSelection() end	end
		SetMenuHiddenCallback(self.unlockSelectionCallback)
		self:LockSelection()
	end
end

-------------------------------------------------------------------------------
-- CONTROL IMPORT
-------------------------------------------------------------------------------

function GodSendPlusImport:New(control)
	local manager = ZO_Object.New(self)
    manager.control = control

    control.container = manager
    
    manager.body = control:GetNamedChild("BodyImport")
    manager.body:SetMaxInputChars(2000)
	
	return manager
end

-------------------------------------------------------------------------------
-- CONTROL COMPOSE
-------------------------------------------------------------------------------

function GodSendPlusCompose:New(control)
    local manager = ZO_Object.New(self)
	manager.control = control

    control.container = manager
        
    manager.subject = control:GetNamedChild("SubjectField")
	manager.subject:SetText("(No subject)")
    manager.subject:SetMaxInputChars(MAIL_MAX_SUBJECT_CHARACTERS)
        
    manager.body = control:GetNamedChild("BodyField")
    manager.body:SetMaxInputChars(1000)
    -- MAIL_MAX_BODY_CHARACTERS)

-- Guild Dropdown
	manager.guildList = ZO_ComboBox_ObjectFromContainer(GetControl(control, "GuildDropdown"))
	manager.guildList:SetSortsItems(false) -- Don't sort list alphabetically
	manager.guildList:SetSpacing(4)
	manager.guildList:ClearItems()
	
-- Logic Dropdown
	manager.logicList = ZO_ComboBox_ObjectFromContainer(GetControl(control, "LogicDropdown"))
	manager.logicList:SetSortsItems(false)
	manager.logicList:SetSpacing(4)
	manager.logicList:ClearItems()
	
-- Rank Dropdown
	manager.rankList = ZO_ComboBox_ObjectFromContainer(GetControl(control, "RankDropdown"))
	manager.rankList:SetSortsItems(false)
	manager.rankList:SetSpacing(4)
	manager.rankList:ClearItems()
	
	local entry
	local guildId
	local guildName
	local rankName
	
	local guildChoice
	local rankChoice
	local logicChoice
	local guildChoiceID
	local rankChoiceID
	
	local function OnRankSelected(_, rankName, choice)
        if not guildChoiceID then
            return
        end

        GodSendPlus_CheckComposeButton (true)
		rankChoice = manager.rankList:GetSelectedItem()
		logicChoice = manager.logicList:GetSelectedItem()
		
		GodSendPlusRecipientsLastRecipient:SetText("")
		GodSendPlusRecipientsProgressBar:SetDimensions(0, 24)
		GodSendPlusRecipientsCancelProgressBar:SetDimensions(0, 24)
		
		Guilds.GuildMembers = {}
		CurrentMail["recipients"] = {}
		
		progressBarTotal = 0
		progressBarUnit = 0
		sentMailCount = 0
		recipientID = 1

		for numGuildMembers = 1, GetNumGuildMembers(guildChoiceID) do
			local name, note, rankIndex, playerStatus, secsSinceLogoff = GetGuildMemberInfo(guildChoiceID, numGuildMembers)
			local memberRank = GetFinalGuildRankName(guildChoiceID, rankIndex)
			local hasCharacter, characterName, zoneName, classId, alliance, level, veteranRank = GetGuildMemberCharacterInfo(guildChoiceID, numGuildMembers)
			table.insert(Guilds.GuildMembers, { id = 0, name = name, rankIndex = rankIndex, rank = memberRank, mail = false }) -- Populate guild members table
		end

   		for k, v in ipairs(Guilds.GuildRanks) do
			if rankChoice == v then rankChoiceID = k else end -- Set rank ID for selected rank
		end

        if userImportMode then
            for k, v in ipairs(Guilds.GuildMembers) do
                v["mail"] = false

                if userImportList[v["name"]:lower()] == true then
                    v["mail"] = true
                end
            end
        else
            for k, v in ipairs(Guilds.GuildMembers) do
                if (v["rankIndex"] == rankChoiceID or rankChoice == "All Members") then v["mail"] = true
                elseif logicChoice == ">=" then
                    if v["rankIndex"] < rankChoiceID then v["mail"] = true
                    else v["mail"] = false end
                elseif logicChoice == "<=" then
                    if v["rankIndex"] > rankChoiceID then v["mail"] = true
                    else v["mail"] = false end
                else v["mail"] = false end
            end
        end
			
		for i, d in ipairs(Guilds.GuildRanks) do
			for k, v in ipairs(Guilds.GuildMembers) do
				if v["rank"] == d then
					if v["mail"] == true then
						v["mail"] = "|c3A92FFPending"
						v["id"] = recipientID
						recipientID = recipientID + 1
						table.insert(CurrentMail["recipients"], v["name"]) -- Populate recipients table
					else end
				else end
			end
		end
		
		recipientID = 1
		GODSENDPLUS_RECIPIENTS:RefreshData() -- Refresh scroll list
		GodSendPlusRecipientsProgressBG:SetHidden(false)
		GodSendPlusRecipientsTotalRecipients:SetColor(hex2rgb("#3A92FF"))
		GodSendPlusRecipientsTotalRecipients:SetText("0/" .. #CurrentMail["recipients"])
		if (GODSENDPLUS_DEBUG) then d(guildChoice .. " (" .. #CurrentMail["recipients"] .. ")") end
		
		progressBarUnit = (500 / #CurrentMail["recipients"])
		if (GODSENDPLUS_DEBUG) then d("Progress Bar Unit = " .. progressBarUnit) end
	end

    _onrs = OnRankSelected
	
	local function OnGuildSelected(_, guildName, choice)

		Guilds.GuildRanks = {}
		manager.logicList:ClearItems()
		manager.rankList:ClearItems()
	
		entry = manager.logicList:CreateItemEntry("==", OnRankSelected)
		manager.logicList:AddItem(entry)
		entry = manager.logicList:CreateItemEntry(">=", OnRankSelected)
		manager.logicList:AddItem(entry)
		entry = manager.logicList:CreateItemEntry("<=", OnRankSelected)
		manager.logicList:AddItem(entry)
		manager.logicList:SetSelectedItem("==")
	
		entry = manager.rankList:CreateItemEntry("All Members", OnRankSelected)
		manager.rankList:AddItem(entry)
		manager.rankList:SetSelectedItem("All Members") -- Set default rank selection
		
		guildChoice = guildName -- Set selected guild name (for debug)
		guildChoiceID = GuildChoice[guildName] -- Set selected guild ID
			
		for numGuildRanks = 1, GetNumGuildRanks(guildChoiceID) do
			rankName = GetFinalGuildRankName(guildChoiceID, numGuildRanks)
			
			entry = manager.rankList:CreateItemEntry(rankName, OnRankSelected) -- Populate rank dropdown box
			manager.rankList:AddItem(entry)
			
			table.insert(Guilds.GuildRanks, rankName) -- Populate guild ranks table
		end
		OnRankSelected()
	end

	for guildIndex = 1, GetNumGuilds() do
		guildId = GetGuildId(guildIndex)
		guildName = GetGuildName(guildId)
		GuildChoice[guildName] = guildId
		
		entry = manager.guildList:CreateItemEntry(guildName, OnGuildSelected) -- Populate guild dropdown box
		manager.guildList:AddItem(entry)
		
		table.insert(Guilds.GuildNames, guildName) -- Populate guild names table
	end
end

-------------------------------------------------------------------------------
-- HANDLERS
-------------------------------------------------------------------------------

-- Mail

local function SaveMailAsPending()
	PendingMail["recipients"] = CurrentMail["recipients"]
	PendingMail["subject"] = CurrentMail["subject"]
	PendingMail["body"] = CurrentMail["body"]
end

local function SendNextRecipient()
	local recipient = nil
	local subject = PendingMail["subject"]
	local body = PendingMail["body"]
	
	GODSENDPLUS_ACTIVE = true
	recipient = PendingMail["recipients"][recipientID]
		
	if not (mailBoxOpen) then RequestOpenMailbox() end
	
	if (mailBoxOpen) then
        SendMail(recipient, subject, body)
        lastKnownRecipient = recipient
	elseif not (mailBoxOpen) then
		if not (GODSENDPLUS_PAUSED) then
		zo_callLater(SendNextRecipient, throttleTimer)
		else GODSENDPLUS_ACTIVE = false 
			CloseMailbox() 
		end
	end

	--if (mailBoxOpen) then
    --    SendMail(recipient, subject, body)
    --    lastKnownRecipient = recipient
	--elseif not (mailBoxOpen) then
	--	if not (GODSENDPLUS_PAUSED) then
	--	zo_callLater(SendNextRecipient, throttleTimer)
	--	else GODSENDPLUS_ACTIVE = false 
	--		CloseMailbox() 
	--	end
	--end
end

local function GodSendPlusCancel()

	if (GODSENDPLUS_ACTIVE) then
	zo_callLater(GodSendPlusCancel, 200)
	else GODSENDPLUS_PAUSED = false
	
		for k, v in ipairs(Guilds.GuildMembers) do
			if v["mail"] == "|c3A92FFPending" then
				v["mail"] = "|cC80F14Canceled"
			else end
		end
		
		local progressBarCancelSize = (500 - progressBarTotal)
		GodSendPlusRecipientsCancelProgressBar:SetDimensions(progressBarCancelSize, 24)
		GodSendPlusRecipientsTotalRecipients:SetColor(hex2rgb("#AFAFAF"))
		GodSendPlusRecipientsLastRecipient:SetText("")
		GODSENDPLUS_RECIPIENTS:RefreshData() -- Refresh scroll list
	end
end

local function UpdateProgressBar()
	progressBarTotal = progressBarTotal + progressBarUnit
	GodSendPlusRecipientsProgressBar:SetDimensions(progressBarTotal, 24)
    if PendingMail and PendingMail["recipients"] then
    	GodSendPlusRecipientsTotalRecipients:SetText(recipientID .. "/" .. #PendingMail["recipients"])
    else
    	GodSendPlusRecipientsTotalRecipients:SetText(recipientID .. "/" .. #CurrentMail["recipients"])
    end
	GodSendPlusRecipientsLastRecipient:SetText(lastKnownRecipient)
end

-- Events

local function OnMailSendSuccess()
	if (GODSENDPLUS_ACTIVE) then
		sentMailCount = sentMailCount + 1
		
		for k, v in ipairs(Guilds.GuildMembers) do
			if v["name"] == lastKnownRecipient then
				v["mail"] = "|c2DC50ESent"
			else end
		end
		
		GodSendPlusRecipientsLastRecipient:SetColor(hex2rgb("#2DC50E"))
		UpdateProgressBar()
		GODSENDPLUS_RECIPIENTS:RefreshData() -- Refresh scroll list
		if (GODSENDPLUS_DEBUG) then d("|c2DC50E" .. lastKnownRecipient .. " " .. recipientID .. "/" .. #PendingMail["recipients"]) end
	
		if recipientID < #PendingMail["recipients"] then
			recipientID = recipientID + 1
			
			if not (GODSENDPLUS_PAUSED) then
			zo_callLater(SendNextRecipient, throttleTimer)
			else GODSENDPLUS_ACTIVE = false 
				CloseMailbox() 
			end
			
		else d("GodSendPlus Completed")
			GODSENDPLUS_ACTIVE = false
			CloseMailbox()
			GodSendPlusHideButtons()
			GodSendPlusRecipientsTotalRecipients:SetColor(hex2rgb("#FFFFFF"))
			GodSendPlusRecipientsLastRecipient:SetColor(hex2rgb("#EECA2A"))
			GodSendPlusRecipientsLastRecipient:SetText("GodSendPlus Completed")
		end
		
	else end
end

local function GodSendPlusStartFrom(playerName, rawName)
    if (GODSENDPLUS_ACTIVE) then
        CHAT_SYSTEM:AddMessage("Can't skip once we've already started! Stop it and try again.")
        return
    end

    local skip_count = 0

    local skip_these = {}

    for i = 1, #CurrentMail["recipients"] do
        local v = CurrentMail["recipients"][i]
        if v == rawName then
            recipientID = i
            skip_count = i
            break
        else
            skip_these[v] = true
        end
    end

    for k, v in ipairs(Guilds.GuildMembers) do
        if skip_these[v["name"]] then
            if v["mail"] ~= false then
                v["mail"] = "|c8b0000Skipped"
            end
        end
    end

    CHAT_SYSTEM:AddMessage("If you start from the wrong person, switch the ranks to rebuild the list, as selecting someone higher up will not reset it.")

	progressBarTotal = progressBarTotal + (progressBarUnit * (skip_count-1))

    UpdateProgressBar()
    GODSENDPLUS_RECIPIENTS:RefreshData() -- Refresh scroll list
end

local function OnMailSendFailed(eventCode, reason)
	if (GODSENDPLUS_ACTIVE) then
		sentMailCount = sentMailCount + 1
		
		for k, v in ipairs(Guilds.GuildMembers) do
			if v["name"] == lastKnownRecipient then
			
				if reason == 8		then v["mail"] = "|cC80F14COD Error"
				elseif reason == 11 then v["mail"] = "|cC80F14Self"
				elseif reason == 7  then v["mail"] = "|cC80F14Blank Mail"
				elseif reason == 1  then v["mail"] = "|cC80F14DB Error"
				elseif reason == 4  then v["mail"] = "|cC80F14Ignored"
				elseif reason == 10 then v["mail"] = "|cC80F14In Progress"
				elseif reason == 2  then v["mail"] = "|cC80F14Invalid Name"
				elseif reason == 3  then v["mail"] = "|cC80F14Full Inbox"
				elseif reason == 6  then v["mail"] = "|cC80F14Invalid Item"
				elseif reason == 12 then v["mail"] = "|cC80F14Mail Disabled"
				elseif reason == 13 then v["mail"] = "|cC80F14Mailbox Closed"
				elseif reason == 9  then v["mail"] = "|cC80F14COD Error"
				elseif reason == 5  then v["mail"] = "|cC80F14Gold Error"
				elseif reason == 15 then v["mail"] = "|cC80F14User Not Found"
				elseif reason == 0  then v["mail"] = "|cC80F14Success"
				elseif reason == 14 then v["mail"] = "|cC80F14Attachment Error"
				else v["mail"] = "|cC80F14Unknown Error"
				end
				
			else end
		end

		GodSendPlusRecipientsLastRecipient:SetColor(hex2rgb("#C80F14"))
		UpdateProgressBar()
		GODSENDPLUS_RECIPIENTS:RefreshData() -- Refresh scroll list
		if (GODSENDPLUS_DEBUG) then d("|cC80F14" .. lastKnownRecipient .. " " .. recipientID .. "/" .. #PendingMail["recipients"]) end
	
		if recipientID < #PendingMail["recipients"] then
			recipientID = recipientID + 1
			
			if not (GODSENDPLUS_PAUSED) then
			zo_callLater(SendNextRecipient, throttleTimer)
			else GODSENDPLUS_ACTIVE = false 
				CloseMailbox()
			end
			
		else d("GodSendPlus Completed")
			GODSENDPLUS_ACTIVE = false
			CloseMailbox()
			GodSendPlusHideButtons()
			GodSendPlusRecipientsTotalRecipients:SetColor(hex2rgb("#FFFFFF"))
			GodSendPlusRecipientsLastRecipient:SetColor(hex2rgb("#EECA2A"))
			GodSendPlusRecipientsLastRecipient:SetText("GodSendPlus Completed")
		end
		
	else end
end

local function OnMailOpenMailBox()
	mailBoxOpen = true
	if (GODSENDPLUS_DEBUG) then d("|c2DC50EMail Box Opened!") end
end

local function OnMailCloseMailBox()
	mailBoxOpen = false
	if (GODSENDPLUS_DEBUG) then d("|cC80F14Mail Box Closed!") end
end

-- Right-Click Menu

function GodSendPlusPlayerContextMenu(playerName, rawName)
	origPlayerContextMenu(playerName, rawName)
    if GODSENDPLUS_WINDOW.visible then
    	AddMenuItem('[GodSendPlus] Start From', function () GodSendPlusStartFrom(playerName, rawName) end)
    end
	ShowMenu()
end	

-- Tooltips

local function SetToolTip(ctrl, text, placement)
    ctrl:SetHandler("OnMouseEnter", function(self)
        ZO_Tooltips_ShowTextTooltip(self, placement, text)
    end)
    ctrl:SetHandler("OnMouseExit", function(self)
        ZO_Tooltips_HideTextTooltip()
    end)
end

-- Keybinds

function GodSendPlusWindow_Toggle()
	GODSENDPLUS_WINDOW:Toggle()
end

-------------------------------------------------------------------------------
-- HOOKS
-------------------------------------------------------------------------------

local function InstallHooks()
	origPlayerContextMenu = CHAT_SYSTEM.ShowPlayerContextMenu
	CHAT_SYSTEM.ShowPlayerContextMenu = GodSendPlusPlayerContextMenu
end
	
-------------------------------------------------------------------------------
-- GLOBAL XML
-------------------------------------------------------------------------------

-- Buttons

function GodSendPlusHideButtons()
	GodSendPlusComposeSendButton:SetHidden(true)
	GodSendPlusComposeCancelButton:SetHidden(true)
	GodSendPlusComposePauseButton:SetHidden(true)
	GodSendPlusComposeContinueButton:SetHidden(true)
end

function GodSendPlusWindowCloseButton_OnClicked()
	GODSENDPLUS_WINDOW.visible = false
	GODSENDPLUS_WINDOW.control:SetHidden(true)
end

function GodSendPlusComposeSendButton_OnClicked()
	SaveMailAsPending()
	SendNextRecipient()

	GodSendPlusRecipientsTotalRecipients:SetColor(hex2rgb("#2DC50E"))
	GodSendPlusComposeSendButton:SetHidden(true)
	GodSendPlusComposeCancelButton:SetHidden(false)
	GodSendPlusComposePauseButton:SetHidden(false)
	if (GODSENDPLUS_DEBUG) then d("GodSendPlus Started") end
end

function GodSendPlusComposeCancelButton_OnClicked()
	GODSENDPLUS_PAUSED = true
	GodSendPlusCancel()
	
	GodSendPlusComposeSendButton:SetHidden(true)
	GodSendPlusComposeCancelButton:SetHidden(true)
	GodSendPlusComposePauseButton:SetHidden(true)
	GodSendPlusComposeContinueButton:SetHidden(true)
	if (GODSENDPLUS_DEBUG) then d("GodSendPlus Canceled") end
end

function GodSendPlusComposePauseButton_OnClicked()
	GODSENDPLUS_PAUSED = true
	
	GodSendPlusRecipientsTotalRecipients:SetColor(hex2rgb("#3A92FF"))
	GodSendPlusComposePauseButton:SetHidden(true)
	GodSendPlusComposeContinueButton:SetHidden(false)
	if (GODSENDPLUS_DEBUG) then d("GodSendPlus Paused") end
end

function GodSendPlusComposeContinueButton_OnClicked()
	GODSENDPLUS_PAUSED = false
	SendNextRecipient()
	
	GodSendPlusRecipientsTotalRecipients:SetColor(hex2rgb("#2DC50E"))
	GodSendPlusComposePauseButton:SetHidden(false)
	GodSendPlusComposeContinueButton:SetHidden(true)
	if (GODSENDPLUS_DEBUG) then d("GodSendPlus Continued") end
end

-- Subject Field

function GodSendPlusComposeSubjectField_OnInitialized(self)
	GodSendPlusComposeSubjectField:SetColor(hex2rgb("#AFAFAF"))
end

function GodSendPlusComposeSubjectField_OnTextChanged(self)
	CurrentMail["subject"] = GodSendPlusComposeSubjectField:GetText()
end

function GodSendPlusComposeSubjectField_OnFocusGained(self)
	if CurrentMail["subject"] == "(No subject)" then
		GodSendPlusComposeSubjectField:SetText("")
	end
	GodSendPlusComposeSubjectField:SetColor(hex2rgb("#FFFFFF"))
	PlaySound(SOUNDS.EDIT_CLICK)
end

function GodSendPlusComposeSubjectField_OnFocusLost(self)
	if CurrentMail["subject"] == "" then
		GodSendPlusComposeSubjectField:SetText("(No subject)")
		GodSendPlusComposeSubjectField:SetColor(hex2rgb("#AFAFAF"))
	end
end

-- Body Field

function GodSendPlus_CheckComposeButton (rank_selected)
    if rank_selected == true then
        GodSendPlusComposeSendButton.rank_selected = true
    end

    if GodSendPlusComposeSendButton.rank_selected == true then
        if CharCounter(CurrentMail["body"]) <= 700 then
            GodSendPlusComposeSendButton:SetHidden(false)
        else
            GodSendPlusComposeSendButton:SetHidden(true)
        end
    end
end


function GodSendPlusComposeBodyField_OnTextChanged(self)
    -- To get around an endless loop of OnTextChanged calls
    -- I do kinda wish there was a RawSetText function that
    -- didn't call handlers.
    if self.modifying and self.modifying == true then return end

    self.modifying = true

    local body = GodSendPlusComposeBodyField:GetText()

    local cursor = GodSendPlusComposeBodyField:GetCursorPosition()

    -- Replacements!
    local repl = {
        ["“"] = '"',    -- Curly quotes open
        ["”"] = '"',    -- Curly quotes close
        ["‘"] = "'",    -- Single curled quote open
        ["’"] = "'",    -- Single curled quote close
        ["\r"] = "",    -- Carriage return
    }

    for k, v in pairs(repl) do
        body = body:gsub(k, v)
    end

    GodSendPlusComposeBodyField:SetText(body)

    GodSendPlusComposeBodyField:SetCursorPosition(cursor)

	CurrentMail["body"] = body

	local CharCountBody = CharCounter(CurrentMail["body"])
	GodSendPlusComposeCharacterLimit:SetText(CharCountBody .. "/700")
    if CharCountBody > 700 then
        GodSendPlusComposeCharacterLimit:SetColor(hex2rgb("#FF0000"))
    else
        GodSendPlusComposeCharacterLimit:SetColor(hex2rgb("#ffffff"))
    end

    GodSendPlus_CheckComposeButton()
    
    self.modifying = false
end

-- Import users

function GodSendPlusImportField_OnTextChanged(self)
    local user_import = GodSendPlusImportBodyImport:GetText()

    if user_import:gsub("\r", ""):gsub("\n", "") == "" then
        userImportMode = false
    else
        local data = {zo_strsplit("\n", user_import)}

        if #data == 1 then
            local data = {zo_strsplit(",", user_import)}
        end

        userImportList = {}

        for i=1, #data do
            local v = data[i]:lower():gsub("\r", ""):gsub("\n", "")

            if v:match("^@") == nil then
                v = "@" .. v
            end

            userImportList[v] = true
        end

        userImportMode = true
    end



    _onrs(nil, nil, nil)
end

-- Scroll List

function GodSendPlusRecipientsRow_OnMouseEnter(control)	
	GODSENDPLUS_RECIPIENTS:EnterRow(control)
end

function GodSendPlusRecipientsRow_OnMouseExit(control)
	GODSENDPLUS_RECIPIENTS:ExitRow(control)
end

function GodSendPlusInboxRow_OnMouseEnter(control)	
	GODSENDPLUS_INBOX:EnterRow(control)
end

function GodSendPlusInboxRow_OnMouseExit(control)
	GODSENDPLUS_INBOX:ExitRow(control)
end

function GodSendPlusSentRow_OnMouseEnter(control)	
	GODSENDPLUS_SENT:EnterRow(control)
end

function GodSendPlusSentRow_OnMouseExit(control)
	GODSENDPLUS_SENT:ExitRow(control)
end

-- Labels

function GodSendPlusLabelField_OnMouseEnter(control)
	if (control:WasTruncated()) then
		InitializeTooltip(InformationTooltip, control, BOTTOM, 0, 0)
		SetTooltipText(InformationTooltip, control:GetText())
	end
	
	local row = control:GetParent()
	zo_callHandler(row, "OnMouseEnter")
end

function GodSendPlusLabelField_OnMouseExit(control)
	ClearTooltip(InformationTooltip)
	
	local row = control:GetParent()
	zo_callHandler(row, "OnMouseExit")
end

function GodSendPlusLabelField_OnLinkMouseUp(control, button, linkText)
	ZO_LinkHandler_OnLinkMouseUp(linkText, button, control)
	
	local row = control:GetParent()
	zo_callHandler(row, "OnMouseUp", button, true, linkText)
end

-- Initialize

function GodSendPlusWindow_OnInitialized(self)
	GODSENDPLUS_WINDOW = GodSendPlusWindow:New(self)
end

function GodSendPlusCompose_OnInitialized(self)
    GODSENDPLUS_COMPOSE = GodSendPlusCompose:New(self)
end

function GodSendPlusRecipients_OnInitialized(self)
    GODSENDPLUS_RECIPIENTS = GodSendPlusRecipients:New(self)
end

function GodSendPlusInbox_OnInitialized(self)
    GODSENDPLUS_INBOX = GodSendPlusInbox:New(self)
end

function GodSendPlusSent_OnInitialized(self)
    GODSENDPLUS_SENT = GodSendPlusSent:New(self)
end

function GodSendPlusImport_OnInitialized(self)
    GODSENDPLUS_IMPORT = GodSendPlusImport:New(self)
end

-------------------------------------------------------------------------------
-- ADDON LOADED
-------------------------------------------------------------------------------

local function OnAddonLoaded(eventCode, addonName)
	if addonName ~= (ADDON_NAME) then return end
	
	SavedVars = ZO_SavedVars:NewAccountWide(SAVEDVARS_NAME, SAVEDVARS_VERSION, nil, DefaultVars)
	
	EVENT_MANAGER:RegisterForEvent(ADDON_NAME, EVENT_MAIL_OPEN_MAILBOX, OnMailOpenMailBox)
	EVENT_MANAGER:RegisterForEvent(ADDON_NAME, EVENT_MAIL_CLOSE_MAILBOX, OnMailCloseMailBox)
	EVENT_MANAGER:RegisterForEvent(ADDON_NAME, EVENT_MAIL_SEND_SUCCESS, OnMailSendSuccess)
	EVENT_MANAGER:RegisterForEvent(ADDON_NAME, EVENT_MAIL_SEND_FAILED, OnMailSendFailed)
	
	SLASH_COMMANDS["/g++"] = GodSendPlusWindow_Toggle
	SLASH_COMMANDS["/g+"] = GodSendPlusWindow_Toggle
	SLASH_COMMANDS["/godsendplus"] = GodSendPlusWindow_Toggle
	
	InstallHooks()
	
	EVENT_MANAGER:UnregisterForEvent(ADDON_NAME, EVENT_ADD_ON_LOADED)
end

EVENT_MANAGER:RegisterForEvent(ADDON_NAME, EVENT_ADD_ON_LOADED, OnAddonLoaded)
